import importlib
import os
import sys

if 'SLACK_RTM_BOT_SETTINGS_FILE' not in os.environ:
  from . import settings_local as settings
else:
  dirname, basename = os.path.split(os.environ['SLACK_RTM_BOT_SETTINGS_FILE'])
  if dirname and dirname not in sys.path:
    sys.path.append(dirname)
  settings = importlib.import_module(os.path.splitext(basename)[0])

globals().update(settings.__dict__)



# -*- coding: utf-8 -*-

from .base import MessageHandler

class TableFlipHandler(MessageHandler):

  TRIGGER_ANCHOR = ''
  TRIGGER_PREFIX = ''
  TRIGGERS = ['tableflip']
  HELP = 'flip a table'

  def handle_message(self, event, triggers, query):
    return '(╯°□°)╯︵ ┻━┻'



from .base import MessageHandler
from .. import settings

class OwnerHandler(MessageHandler):

  TRIGGER_ANCHOR = ''
  TRIGGER_PREFIX = '@'
  TRIGGERS = ['owner', 'owners']
  HELP = 'notify channel owner(s)'

  _OWNERS = None

  def handle_message(self, event, triggers, query):
    if self._OWNERS is None:
      self._init_owners()

    return self._OWNERS.get(event['channel'])

  def _init_owners(self):
    self._OWNERS = {
        channel: ' '.join('@%s' % owner for owner in sorted(owners))
        for channel, owners in settings.CHANNEL_OWNERS.items()}



from setuptools import setup

version = {}

with open('slack_rtm_bot/__version__.py', 'r') as f:
  exec(f.read(), version)

with open('README.md', 'r') as f:
  readme = f.read()

setup(
    name='slack-rtm-bot',
    version=version['__version__'],
    description='Slack RTM bot',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/nkouevda/slack-rtm-bot',
    author='Nikita Kouevda',
    author_email='nkouevda@gmail.com',
    license='MIT',
    packages=[
        'slack_rtm_bot',
        'slack_rtm_bot.handlers',
    ],
    install_requires=[
        'requests',
        'six',
        'slackclient',
        'websocket-client',
    ],
    entry_points={
        'console_scripts': [
            'slack-rtm-bot=slack_rtm_bot.bot:main',
        ],
    },
)



from collections import Counter
import random

from .base import MessageHandler
from ..util import parse_int

class DiceHandler(MessageHandler):

  TRIGGERS = ['dice', 'roll']
  HELP = 'roll the given number of dice; default 1'

  def handle_message(self, event, triggers, query):
    times = parse_int(query, default=1)
    dice_gen = (random.randrange(1, 7) for _ in xrange(times))
    if times > 10:
      counter = Counter(dice_gen)
      return ', '.join('%s: %s' % (k, counter[k]) for k in xrange(1, 7))
    else:
      return ', '.join(map(str, dice_gen))



import re

from .base import handlers, MessageHandler

class HelpHandler(MessageHandler):

  TRIGGERS = ['help']
  HELP = 'help for the given command; default all commands'

  _RESPONSES = None

  def handle_message(self, event, triggers, query):
    if self._RESPONSES is None:
      self._init_responses()

    parts = []
    for trigger in re.findall(r'\w+', query.lower()):
      if trigger in self._RESPONSES:
        response = self._RESPONSES[trigger]
        if response not in parts:
          parts.append(response)

    return '```%s```' % ('\n'.join(parts) if parts else self._RESPONSES[None])

  def _init_responses(self):
    self._RESPONSES = {}

    for handler in handlers:
      if handler.TRIGGERS and handler.HELP:
        triggers = ', '.join('%s%s' % (
            handler.TRIGGER_PREFIX, trigger) for trigger in handler.TRIGGERS)
        for trigger in handler.TRIGGERS:
          self._RESPONSES[trigger] = '%s - %s' % (triggers, handler.HELP)

    self._RESPONSES[None] = '\n'.join(sorted(set(self._RESPONSES.values())))
